#!/usr/bin/php
<?php
/*
// isdf - dhcp generator
// Author: Carsten Schoene
//
// $LastChangedDate: 2012-09-18 15:27:50 +0200 (Tue, 18 Sep 2012) $
// $Rev: 1857 $
*/

// define our name
define('MYNAME', "create_dhcp_configs");
// open syslog connection
openlog(MYNAME,LOG_PID | LOG_ODELAY,LOG_MAIL);

// check for required extensions
if ( ! extension_loaded("mysql") ) {
        if ( ! dl("mysql") ) {
                syslog(LOG_ERR,"mysql extension not loaded!");
                exit;
        }
}

// load additional files
define('BASE',dirname(__FILE__));
require_once("MDB2.php");
require_once(BASE . "/config.inc.php");
require_once(BASE . "/config-custom.inc.php");
require_once(BASE . "/db.inc.php");
require_once(BASE . "/inet4.class.php");
require_once(BASE . "/inv-dhcp4.class.php");
require_once(BASE . "/dhcpconf.class.php");

// create database connection
$dbh =& MDB2::singleton($config['isdf']['db'], $config['mdb2']['options'] );

// create inv-dhcp4 object
$idhcp4 = new inv_dhcp4();

// create dhcpconf objects
if ( is_file($config["isdf"]["dhcp"]["system"]["net4file"])) {
	$dhcpsystem = new dhcpconf($config["isdf"]["dhcp"]["system"]["net4file"]);
} else {
	die("File " . $config["isdf"]["dhcp"]["system"]["net4file"] . " not found!");
}
if ( is_file($config["isdf"]["dhcp"]["systemmgmt"]["net4file"])) {
	$dhcpsystemmgmt = new dhcpconf($config["isdf"]["dhcp"]["systemmgmt"]["net4file"]);
} else {
	die("File " . $config["isdf"]["dhcp"]["systemmgmt"]["net4file"] . " not found!");
}
if ( is_file($config["isdf"]["dhcp"]["systempxe"]["net4file"])) {
	$dhcpsystempxe = new dhcpconf($config["isdf"]["dhcp"]["systempxe"]["net4file"]);
} else {
	die("File " . $config["isdf"]["dhcp"]["systempxe"]["net4file"] . " not found!");
}

// create inet4 objects
$systemaddr = explode("/", $config["isdf"]["dhcp"]["system"]["net4"]);
$systeminet4 = new IPv4($systemaddr[0], $systemaddr[1]);
$pxeaddr = explode("/", $config["isdf"]["dhcp"]["systempxe"]["net4"]);
$pxeinet4 = new IPv4($pxeaddr[0], $pxeaddr[1]);

$systemend = $systeminet4->broadcast();
$pxeend = $pxeinet4->broadcast();

// create dhcp entry for host & pxe 
$qrybkvm = "SELECT t1.value AS value FROM inv_system_properties t1, inv_system_properties t2 WHERE t1.datakey='get_system_net_info' AND t1.inv_system_id=t2.inv_system_id AND t2.datakey='" . $config["isdf"]["dhcp"]["db_datakey"] . "' AND t2.value='" . $config["isdf"]["dhcp"]["db_datavalue"] . "';";

$dhcpmacs = array();

$res = $dbh->query($qrybkvm);
if (PEAR::isError($res)) {
        syslog(LOG_ERR, "MDB2: " . $res->getMessage());
} else {
        while (( $row = $res->fetchRow(MDB2_FETCHMODE_ASSOC) )) {
		// each result is interface list with values of mac,interface,linkstatus
		$intlist = preg_split('/\s/',$row['value']);
		foreach ( $intlist as $macintlink ) {
			list($mac,$int,$link) = explode(",", trim($macintlink));
			if ($idhcp4->int_is_valid($mac,$int,$link)) {
				// create dhcp entry - push to array
				array_push($dhcpmacs, $mac);
			}
		}
	}
}
//print_r($dhcpmacs);
foreach ( $dhcpmacs as $mac ) {
	$retval="";
	//echo "Working on MAC: $mac \n";
	$systemfirst = $config["isdf"]["dhcp"]["system"]["firstusable"];
	if ( $systeminet4->isipin($systemfirst) ) {
		$revhost = $config["isdf"]["dhcp"]["system"]["hostnameprefix"] . preg_replace('/\./','-', $systemfirst);
		//echo "  first try on with: $revhost\n";
		$reread = true;
		while ( !($retval = $dhcpsystem->hostExists($revhost,$mac, $reread)) === false ) {
			//echo "Retval: $retval\n";
			if ( $retval == 2 ) {
				// mac exists
				//echo "  mac address found in config\n";
				break 2;
			} elseif ( $retval == 1 ) {
				// hostname/ip exists
				//echo "  another try with: $revhost\n";
				$systemfirst = $systeminet4->increment($systemfirst,$systemend);
				$revhost = $config["isdf"]["dhcp"]["system"]["hostnameprefix"] . preg_replace('/\./', '-', $systemfirst);
				$reread = false;
			} else {
				break;
			}
		}
		//echo " while loop ended with: $revhost\n";
		$dhcpsystem->addHost($revhost,$mac,$systemfirst);
	}
}
foreach ( $dhcpmacs as $mac ) {
	$retval="";
	$pxefirst = $config["isdf"]["dhcp"]["systempxe"]["firstusable"];
	if ( $pxeinet4->isipin($pxefirst) ) {
		$revhost = $config["isdf"]["dhcp"]["systempxe"]["hostnameprefix"] . preg_replace('/\./','-', $pxefirst);
		$reread = true;
		while ( !($retval = $dhcpsystempxe->hostExists($revhost,$mac, $reread)) === false ) {
			if ( $retval == 2 ) {
				break 2;
			} elseif ( $retval == 1 ) {
				$pxefirst = $pxeinet4->increment($pxefirst,$pxeend);
				$revhost = $config["isdf"]["dhcp"]["systempxe"]["hostnameprefix"] . preg_replace('/\./', '-', $pxefirst);
				$reread = false;
			} else {
				break;
			}
		}
		$dhcpsystempxe->addHost($revhost,$mac,$pxefirst);
	}
}



// create dhcp entry for mgmt
$qrymgmtbkvm = "SELECT t1.value AS value FROM inv_system_properties t1, inv_system_properties t2 WHERE t1.datakey='get_system_mgmt_mac' AND t1.inv_system_id=t2.inv_system_id AND t2.datakey='" . $config["isdf"]["dhcp"]["db_datakey"] . "' AND t2.value='" . $config["isdf"]["dhcp"]["db_datavalue"] . "';";

$mgmtaddr = explode("/", $config["isdf"]["dhcp"]["systemmgmt"]["net4"]);
$mgmtinet4 = new IPv4($mgmtaddr[0], $mgmtaddr[1]);
$mgmtend = $mgmtinet4->broadcast();

$dhcpmgmtmacs = array();

$res = $dbh->query($qrymgmtbkvm);
if (PEAR::isError($res)) {
	syslog(LOG_ERR, "MDB2: " . $res->getMessage());
} else {
	while (( $row = $res->fetchRow(MDB2_FETCHMODE_ASSOC) )) {
		// each result is interface list with values of mac,interface,linkstatus
		$mac = trim($row['value']);
		if ($idhcp4->mac_is_valid($mac)) {
			// create dhcp entry - push to array
			array_push($dhcpmgmtmacs, $mac);
		}
	}
}

foreach ( $dhcpmgmtmacs as $mac ) {
	$retval="";
	$mgmtfirst = $config["isdf"]["dhcp"]["systemmgmt"]["firstusable"];
	if ( $mgmtinet4->isipin($mgmtfirst) ) {
		$revhost = $config["isdf"]["dhcp"]["systemmgmt"]["hostnameprefix"] . preg_replace('/\./','-', $mgmtfirst);
		$reread = true;
		while ( !($retval = $dhcpsystemmgmt->hostExists($revhost,$mac,$reread)) === false ) {
			if ( $retval == 2 ) {
				break 2;
			} elseif ( $retval == 1 ) {
				$mgmtfirst = $mgmtinet4->increment($mgmtfirst,$mgmtend);
				$revhost = $config["isdf"]["dhcp"]["systemmgmt"]["hostnameprefix"] . preg_replace('/\./', '-', $mgmtfirst);
				$reread = false;
			} else {
				break;
			}
		}
		$dhcpsystemmgmt->addHost($revhost,$mac,$mgmtfirst);
	}
}

$dbh->disconnect();
